# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TagsUpdateResponse(UniversalBaseModel):
    id: str
    name: str = pydantic.Field()
    """
    a string at most 100 character(s) long
    """

    project_id: typing_extensions.Annotated[str, FieldMetadata(alias="projectId")]
    organization_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="organizationId")
    ]
    created_by: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="createdBy")
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="createdAt")
    ] = None
    updated_at: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="updatedAt")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
