# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .subscription_past_due_error_tag import SubscriptionPastDueErrorTag


class SubscriptionPastDueError(UniversalBaseModel):
    message: str
    stripe_customer_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="stripeCustomerId")
    ]
    past_due_subscription_ids: typing_extensions.Annotated[
        typing.List[str], FieldMetadata(alias="pastDueSubscriptionIds")
    ]
    tag: SubscriptionPastDueErrorTag

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
