# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TokenCostCalculateResponse(UniversalBaseModel):
    input_cost_centicents: typing_extensions.Annotated[
        float, FieldMetadata(alias="inputCostCenticents")
    ]
    output_cost_centicents: typing_extensions.Annotated[
        float, FieldMetadata(alias="outputCostCenticents")
    ]
    cache_read_cost_centicents: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheReadCostCenticents")
    ] = None
    cache_write_cost_centicents: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheWriteCostCenticents")
    ] = None
    total_cost_centicents: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalCostCenticents")
    ]
    input_cost_dollars: typing_extensions.Annotated[
        float, FieldMetadata(alias="inputCostDollars")
    ]
    output_cost_dollars: typing_extensions.Annotated[
        float, FieldMetadata(alias="outputCostDollars")
    ]
    cache_read_cost_dollars: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheReadCostDollars")
    ] = None
    cache_write_cost_dollars: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheWriteCostDollars")
    ] = None
    total_cost_dollars: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalCostDollars")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
