# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .functions_list_by_env_response_functions_item import (
    FunctionsListByEnvResponseFunctionsItem,
)


class FunctionsListByEnvResponse(UniversalBaseModel):
    functions: typing.List[FunctionsListByEnvResponseFunctionsItem]
    total: float

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
