# This file was auto-generated by Fern from our API Definition.

import typing

OrganizationMembershipsListResponseItemRole = typing.Union[
    typing.Literal["OWNER", "ADMIN", "MEMBER"], typing.Any
]
