# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .traces_get_trace_detail_response_spans_item import (
    TracesGetTraceDetailResponseSpansItem,
)


class TracesGetTraceDetailResponse(UniversalBaseModel):
    trace_id: typing_extensions.Annotated[str, FieldMetadata(alias="traceId")]
    spans: typing.List[TracesGetTraceDetailResponseSpansItem]
    root_span_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="rootSpanId")
    ] = None
    total_duration_ms: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalDurationMs")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
