"""Functions for testing the `Closure` class."""

from . import other  # re-export submodule for attribute access in tests
from .main import (
    aliased_import_fn,
    aliased_module_import_fn,
    annotated_assignment_fn,
    annotated_container_fn,
    annotated_dependency_fn,
    annotated_input_arg_fn,
    built_in_fn,
    closure_inside_decorator_fn,
    closure_inside_imported_decorator_fn,
    closure_with_long_function_name_that_wraps_around_fn,
    closure_with_properties_fn,
    datetime_fn,
    decorated_fn,
    decorated_with_model_fn,
    dotted_import_fn,
    fakepkg_import_fn,
    fn_inside_class_fn,
    global_var_fn,
    import_with_different_dist_name_fn,
    inner_class_fn,
    inner_fn,
    inner_sub_fn,
    internal_imports_fn,
    kwarg_rich_fn,
    multi_decorated_fn,
    multi_decorator_with_version_fn,
    nested_class_guard_fn,
    ops_version_decorated_empty_parens_fn,
    ops_version_decorated_fn,
    ops_version_decorated_with_tags_fn,
    property_missing_getter_fn,
    registry_decorated_fn,
    self_fn_class_fn,
    single_fn,
    standard_import_fn,
    sub_fn,
    sub_fn_inside_class_fn,
    third_party_fn,
    type_alias_fn,
    user_defined_aliased_class_import_fn,
    user_defined_aliased_dotted_import_fn,
    user_defined_aliased_import_fn,
    user_defined_aliased_module_import_class_fn,
    user_defined_aliased_module_import_fn,
    user_defined_class_from_import_fn,
    user_defined_class_import_fn,
    user_defined_dotted_import_fn,
    user_defined_from_import_fn,
    user_defined_import_fn,
    user_defined_plain_import_fn,
    version_decorated_empty_parens_fn,
    version_decorated_fn,
    version_decorated_with_all_args_fn,
    version_decorated_with_tags_fn,
    version_first_multi_decorator_fn,
)

__all__ = [
    "aliased_import_fn",
    "aliased_module_import_fn",
    "annotated_assignment_fn",
    "annotated_container_fn",
    "annotated_dependency_fn",
    "annotated_input_arg_fn",
    "built_in_fn",
    "closure_inside_decorator_fn",
    "closure_inside_imported_decorator_fn",
    "closure_with_long_function_name_that_wraps_around_fn",
    "closure_with_properties_fn",
    "datetime_fn",
    "decorated_fn",
    "decorated_with_model_fn",
    "dotted_import_fn",
    "fakepkg_import_fn",
    "fn_inside_class_fn",
    "global_var_fn",
    "import_with_different_dist_name_fn",
    "inner_class_fn",
    "inner_fn",
    "inner_sub_fn",
    "internal_imports_fn",
    "kwarg_rich_fn",
    "multi_decorated_fn",
    "multi_decorator_with_version_fn",
    "nested_class_guard_fn",
    "ops_version_decorated_empty_parens_fn",
    "ops_version_decorated_fn",
    "ops_version_decorated_with_tags_fn",
    "other",
    "property_missing_getter_fn",
    "registry_decorated_fn",
    "self_fn_class_fn",
    "single_fn",
    "standard_import_fn",
    "sub_fn",
    "sub_fn_inside_class_fn",
    "third_party_fn",
    "type_alias_fn",
    "user_defined_aliased_class_import_fn",
    "user_defined_aliased_dotted_import_fn",
    "user_defined_aliased_import_fn",
    "user_defined_aliased_module_import_class_fn",
    "user_defined_aliased_module_import_fn",
    "user_defined_class_from_import_fn",
    "user_defined_class_import_fn",
    "user_defined_dotted_import_fn",
    "user_defined_from_import_fn",
    "user_defined_import_fn",
    "user_defined_plain_import_fn",
    "version_decorated_empty_parens_fn",
    "version_decorated_fn",
    "version_decorated_with_all_args_fn",
    "version_decorated_with_tags_fn",
    "version_first_multi_decorator_fn",
]
