# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from .annotations.client import AnnotationsClient, AsyncAnnotationsClient
from .api_keys.client import ApiKeysClient, AsyncApiKeysClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .docs.client import AsyncDocsClient, DocsClient
from .environment import MirascopeEnvironment
from .environments.client import AsyncEnvironmentsClient, EnvironmentsClient
from .functions.client import AsyncFunctionsClient, FunctionsClient
from .health.client import AsyncHealthClient, HealthClient
from .organization_invitations.client import (
    AsyncOrganizationInvitationsClient,
    OrganizationInvitationsClient,
)
from .organization_memberships.client import (
    AsyncOrganizationMembershipsClient,
    OrganizationMembershipsClient,
)
from .organizations.client import AsyncOrganizationsClient, OrganizationsClient
from .project_memberships.client import (
    AsyncProjectMembershipsClient,
    ProjectMembershipsClient,
)
from .projects.client import AsyncProjectsClient, ProjectsClient
from .tags.client import AsyncTagsClient, TagsClient
from .token_cost.client import AsyncTokenCostClient, TokenCostClient
from .traces.client import AsyncTracesClient, TracesClient


class Mirascope:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MirascopeEnvironment
        The environment to use for requests from the client. from .environment import MirascopeEnvironment



        Defaults to MirascopeEnvironment.PRODUCTION



    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 180 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mirascope.api._generated import Mirascope

    client = Mirascope()
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MirascopeEnvironment = MirascopeEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout
            if timeout is not None
            else 180
            if httpx_client is None
            else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.health = HealthClient(client_wrapper=self._client_wrapper)
        self.traces = TracesClient(client_wrapper=self._client_wrapper)
        self.docs = DocsClient(client_wrapper=self._client_wrapper)
        self.organizations = OrganizationsClient(client_wrapper=self._client_wrapper)
        self.organization_invitations = OrganizationInvitationsClient(
            client_wrapper=self._client_wrapper
        )
        self.organization_memberships = OrganizationMembershipsClient(
            client_wrapper=self._client_wrapper
        )
        self.projects = ProjectsClient(client_wrapper=self._client_wrapper)
        self.project_memberships = ProjectMembershipsClient(
            client_wrapper=self._client_wrapper
        )
        self.environments = EnvironmentsClient(client_wrapper=self._client_wrapper)
        self.api_keys = ApiKeysClient(client_wrapper=self._client_wrapper)
        self.functions = FunctionsClient(client_wrapper=self._client_wrapper)
        self.annotations = AnnotationsClient(client_wrapper=self._client_wrapper)
        self.tags = TagsClient(client_wrapper=self._client_wrapper)
        self.token_cost = TokenCostClient(client_wrapper=self._client_wrapper)


class AsyncMirascope:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MirascopeEnvironment
        The environment to use for requests from the client. from .environment import MirascopeEnvironment



        Defaults to MirascopeEnvironment.PRODUCTION



    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 180 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mirascope.api._generated import AsyncMirascope

    client = AsyncMirascope()
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MirascopeEnvironment = MirascopeEnvironment.PRODUCTION,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout
            if timeout is not None
            else 180
            if httpx_client is None
            else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.health = AsyncHealthClient(client_wrapper=self._client_wrapper)
        self.traces = AsyncTracesClient(client_wrapper=self._client_wrapper)
        self.docs = AsyncDocsClient(client_wrapper=self._client_wrapper)
        self.organizations = AsyncOrganizationsClient(
            client_wrapper=self._client_wrapper
        )
        self.organization_invitations = AsyncOrganizationInvitationsClient(
            client_wrapper=self._client_wrapper
        )
        self.organization_memberships = AsyncOrganizationMembershipsClient(
            client_wrapper=self._client_wrapper
        )
        self.projects = AsyncProjectsClient(client_wrapper=self._client_wrapper)
        self.project_memberships = AsyncProjectMembershipsClient(
            client_wrapper=self._client_wrapper
        )
        self.environments = AsyncEnvironmentsClient(client_wrapper=self._client_wrapper)
        self.api_keys = AsyncApiKeysClient(client_wrapper=self._client_wrapper)
        self.functions = AsyncFunctionsClient(client_wrapper=self._client_wrapper)
        self.annotations = AsyncAnnotationsClient(client_wrapper=self._client_wrapper)
        self.tags = AsyncTagsClient(client_wrapper=self._client_wrapper)
        self.token_cost = AsyncTokenCostClient(client_wrapper=self._client_wrapper)


def _get_base_url(
    *, base_url: typing.Optional[str] = None, environment: MirascopeEnvironment
) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception(
            "Please pass in either base_url or environment to construct the client"
        )
