"""Anthropic model information.

This file is auto-generated by scripts/model_features/codegen_anthropic.py
Do not edit manually - run the codegen script to update."""

from typing import Literal

AnthropicKnownModels = Literal[
    "anthropic/claude-3-5-haiku",
    "anthropic/claude-3-5-haiku-20241022",
    "anthropic/claude-3-5-haiku-latest",
    "anthropic/claude-3-7-sonnet",
    "anthropic/claude-3-7-sonnet-20250219",
    "anthropic/claude-3-7-sonnet-latest",
    "anthropic/claude-3-haiku",
    "anthropic/claude-3-haiku-20240307",
    "anthropic/claude-3-haiku-latest",
    "anthropic/claude-3-opus",
    "anthropic/claude-3-opus-20240229",
    "anthropic/claude-3-opus-latest",
    "anthropic/claude-haiku-4-5",
    "anthropic/claude-haiku-4-5-0",
    "anthropic/claude-haiku-4-5-0-20251001",
    "anthropic/claude-haiku-4-5-0-latest",
    "anthropic/claude-haiku-4-5-20251001",
    "anthropic/claude-haiku-4-5-latest",
    "anthropic/claude-opus-4",
    "anthropic/claude-opus-4-0",
    "anthropic/claude-opus-4-0-20250514",
    "anthropic/claude-opus-4-0-latest",
    "anthropic/claude-opus-4-1",
    "anthropic/claude-opus-4-1-0",
    "anthropic/claude-opus-4-1-0-20250805",
    "anthropic/claude-opus-4-1-0-latest",
    "anthropic/claude-opus-4-1-20250805",
    "anthropic/claude-opus-4-1-latest",
    "anthropic/claude-opus-4-20250514",
    "anthropic/claude-opus-4-5",
    "anthropic/claude-opus-4-5-0",
    "anthropic/claude-opus-4-5-0-20251101",
    "anthropic/claude-opus-4-5-0-latest",
    "anthropic/claude-opus-4-5-20251101",
    "anthropic/claude-opus-4-5-latest",
    "anthropic/claude-opus-4-latest",
    "anthropic/claude-sonnet-4",
    "anthropic/claude-sonnet-4-0",
    "anthropic/claude-sonnet-4-0-20250514",
    "anthropic/claude-sonnet-4-0-latest",
    "anthropic/claude-sonnet-4-20250514",
    "anthropic/claude-sonnet-4-5",
    "anthropic/claude-sonnet-4-5-0",
    "anthropic/claude-sonnet-4-5-0-20250929",
    "anthropic/claude-sonnet-4-5-0-latest",
    "anthropic/claude-sonnet-4-5-20250929",
    "anthropic/claude-sonnet-4-5-latest",
    "anthropic/claude-sonnet-4-latest",
]
"""Valid Anthropic model IDs."""


MODELS_WITHOUT_STRICT_STRUCTURED_OUTPUTS: set[str] = {
    "claude-3-5-haiku",
    "claude-3-5-haiku-20241022",
    "claude-3-5-haiku-latest",
    "claude-3-7-sonnet",
    "claude-3-7-sonnet-20250219",
    "claude-3-7-sonnet-latest",
    "claude-3-haiku",
    "claude-3-haiku-20240307",
    "claude-3-haiku-latest",
    "claude-3-opus",
    "claude-3-opus-20240229",
    "claude-3-opus-latest",
    "claude-opus-4",
    "claude-opus-4-0",
    "claude-opus-4-0-20250514",
    "claude-opus-4-0-latest",
    "claude-opus-4-20250514",
    "claude-opus-4-latest",
    "claude-sonnet-4",
    "claude-sonnet-4-0",
    "claude-sonnet-4-0-20250514",
    "claude-sonnet-4-0-latest",
    "claude-sonnet-4-20250514",
    "claude-sonnet-4-latest",
}
"""Models that do not support strict structured outputs (strict mode tools)."""
