# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .functions_list_by_env_response_functions_item_dependencies_value import (
    FunctionsListByEnvResponseFunctionsItemDependenciesValue,
)


class FunctionsListByEnvResponseFunctionsItem(UniversalBaseModel):
    id: str
    hash: str
    signature_hash: typing_extensions.Annotated[
        str, FieldMetadata(alias="signatureHash")
    ]
    name: str
    description: typing.Optional[str] = None
    version: str
    tags: typing.Optional[typing.List[str]] = None
    metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
    code: str
    signature: str
    dependencies: typing.Optional[
        typing.Dict[
            str,
            typing.Optional[FunctionsListByEnvResponseFunctionsItemDependenciesValue],
        ]
    ] = None
    environment_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="environmentId")
    ]
    project_id: typing_extensions.Annotated[str, FieldMetadata(alias="projectId")]
    organization_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="organizationId")
    ]
    created_at: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="createdAt")
    ] = None
    updated_at: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="updatedAt")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
