# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TokenCostCalculateRequestUsage(UniversalBaseModel):
    input_tokens: typing_extensions.Annotated[
        float, FieldMetadata(alias="inputTokens")
    ] = pydantic.Field()
    """
    a non-negative number
    """

    output_tokens: typing_extensions.Annotated[
        float, FieldMetadata(alias="outputTokens")
    ] = pydantic.Field()
    """
    a non-negative number
    """

    cache_read_tokens: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheReadTokens")
    ] = pydantic.Field(default=None)
    """
    a non-negative number
    """

    cache_write_tokens: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="cacheWriteTokens")
    ] = pydantic.Field(default=None)
    """
    a non-negative number
    """

    cache_write_breakdown: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, float]],
        FieldMetadata(alias="cacheWriteBreakdown"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
